'use strict';

import {
	CodeAction,
	CodeActionKind,
	createConnection,
	Diagnostic,
	DiagnosticSeverity,
	InitializeResult,
	ProposedFeatures,
	Range,
	TextDocumentEdit,
	TextDocuments,
	TextDocumentSyncKind,
	TextEdit,
} from 'vscode-languageserver/node';
import { TextDocument } from 'vscode-languageserver-textdocument';

// サーバー接続オブジェクトを作成する。この接続にはNodeのIPC(プロセス間通信)を利用する
// LSPの全機能を提供する
const connection = createConnection(ProposedFeatures.all);
connection.console.info(`Sample server running in node ${process.version}`);
// 初期化ハンドルでインスタンス化する
let documents!: TextDocuments<TextDocument>;

// 接続の初期化
connection.onInitialize((_params, _cancel, progress) => {
	// サーバーの起動を進捗表示する
	progress.begin('Initializing Sample Server');
	// テキストドキュメントを監視する
	documents = new TextDocuments(TextDocument);
	setupDocumentsListeners();
	// 起動進捗表示の終了
	progress.done();

	return {
		// サーバー仕様
		capabilities: {
			// ドキュメントの同期
			textDocumentSync: {
				openClose: true,
				change: TextDocumentSyncKind.Incremental,
				willSaveWaitUntil: false,
				save: {
					includeText: false,
				},
			},
			// ここを追加
			codeActionProvider: {
				codeActionKinds: [CodeActionKind.QuickFix],
				resolveProvider: true,
			},
		},
	} as InitializeResult;
});

/**
 * 大文字に対して警告を表示する
 */
function validate(doc: TextDocument) {
	// エディターで開いているファイルの文字列を取得
	const text = doc.getText();
	// 2つ以上並んでいるアルファベット大文字を検出するための正規表現
	const pattern = /\b[A-Z]{2,}\b/g;
	let m: RegExpExecArray | null;

	// 警告などの状態を管理するリスト
	const diagnostics: Diagnostic[] = [];
	// 正規表現に引っかかった文字列すべてを対象にする
	while ((m = pattern.exec(text)) !== null) {
		// 対象の位置から正規表現に引っかかった文字列までを対象にする
		const range: Range = {
			start: doc.positionAt(m.index),
			end: doc.positionAt(m.index + m[0].length),
		};
		// 警告内容を作成、上から範囲、メッセージ、重要度、ID、警告の出力元
		const diagnostic: Diagnostic = Diagnostic.create(
			range,
			`${m[0]} is all uppercase.`,
			DiagnosticSeverity.Warning,
			'',
			'sample'
		);
		// 警告リストに警告内容を追加
		diagnostics.push(diagnostic);
	}

	// VS Codeに警告リストを送信
	void connection.sendDiagnostics({ uri: doc.uri, diagnostics });
}

/**
 * ドキュメントの動作を監視する
 */
function setupDocumentsListeners() {
	documents.listen(connection);

	documents.onDidOpen((event) => {
		validate(event.document);
	});

	documents.onDidChangeContent((change) => {
		validate(change.document);
	});

	documents.onDidClose((close) => {
		void connection.sendDiagnostics({
			uri: close.document.uri,
			diagnostics: [],
		});
	});

	// ここから追加

	// Code Actionを追加する
	connection.onCodeAction((params) => {
		// sampleから生成した警告のみを対象とする
		const diagnostics = params.context.diagnostics.filter(
			(diag) => diag.source === 'sample'
		);
		// 対象ファイルを取得する
		const textDocument = documents.get(params.textDocument.uri);
		if (textDocument === undefined || diagnostics.length === 0) {
			return [];
		}
		const codeActions: CodeAction[] = [];
		// 各警告に対してアクションを生成する
		diagnostics.forEach((diag) => {
			// アクションの目的
			const title = 'Fix to lower case';
			// 警告範囲の文字列取得
			const originalText = textDocument.getText(diag.range);
			// 該当箇所を小文字に変更
			const edits = [
				TextEdit.replace(diag.range, originalText.toLowerCase())
			];
			const editPattern = {
				documentChanges: [
					TextDocumentEdit.create(
						{ uri: textDocument.uri, version: textDocument.version },
						edits
					),
				],
			};
			// コードアクションを生成
			const fixAction = CodeAction.create(
				title,
				editPattern,
				CodeActionKind.QuickFix
			);
			// コードアクションと警告を関連付ける
			fixAction.diagnostics = [diag];
			codeActions.push(fixAction);
		});

		return codeActions;
	});
}

// Listen on the connection
connection.listen();
